/* 
 * File:   main.c
 * Author: Daniel
 *
 * Created on November 11, 2013, 8:07 PM
 */

#include <xc.h>
#include <stdio.h>
#include "main.h"
#include "config_bits.h"
#include "uart.h"
#include <i2c.h>


#define I2C_BRG 18

unsigned char UART_RX_buffer[UART_RX_BUFFER_SIZE];       // UART receive buffer
unsigned char UART_RX_buffer_size=0;
volatile unsigned char U1RX_flag=0;                        // new data in the RX buffer


int main()
{
    
    Initial();
    //txStr("hello, world!\r");
    

    GPIO2 = 1;
    GPIO0 = 1;
    GPIO1 = 1;

    LED = 1;
    
    while (1) { 
        //if (num++ == 0)
            //LED = !LED;

        if (UART_RX_buffer_size > 0)    // do we have serial data?
        {
            if (UART_RX_buffer[UART_RX_buffer_size-1] == 13 || UART_RX_buffer_size >= UART_RX_BUFFER_SIZE)  // do we have a \r or at the end of our buffer?
            {
                processSerialData();
            }
        }
        Idle();
    }
    
    return 0;
}

void processSerialData()
{
    //UART_RX_buffer[] contains serial data
    //UART_RX_buffer_size contains the number of bytes receieved
    //      There is an extra byte for the \r character used to delimit commands

    unsigned int value;
    
    float temp, humid;
    
    //int ret;

    switch (UART_RX_buffer[0])
    {
        case 'L':   case 'l':
            //txStr("Reading ADC... ");

            value = readLightSensor();
            //txVal("-Light: ", num);
            printf("-Light: %u\r", value);

            break;
        case 'T': case 't':
            LED = 0;
            temp = HTU21D_readTemp();
            LED = 1;
            
            printf("-Temp: %3.1f\r", (double)temp);

            break;
        case 'H': case 'h':
            LED = 0;
            humid = HTU21D_readHumidity();
            LED = 1;

            printf("-Humidity: %3.1f\r", (double)humid);
            break;
            
        case 'Q':
            printf("-Hello, world!\r");
            break;
        case 'q':
            temp = -6;
            printf("%3.1f\r", temp);

            break;

        case 'a':   // output temp, humidity, and light sensor
            txStr("k\r");
            LED = 0;
            temp = HTU21D_readTemp();
            humid = HTU21D_readHumidity();
            value = readLightSensor();
            value = (int)(((double)value / (double)1024) * (double)100);
            if (value > 99) value = 99;
            if ( humid > (float)99.9) humid = 99.9; //max out humidity value for string parsing reasons
            if ( humid < (float)0) humid = 0;
            LED = 1;
            //format is: TTT.T HH.H LL\r
            // T is temp digits, H is humidity digits, L is light digits (0-88ish)


            printf(">E%5.1f %3.1f %02u\r", (double)temp, (double)humid, value);

            break;

        case 'P': case 'p':

             break;
        case '-': case '!': //ignore
            break;
            
        default:
            bad_command:
            txStr("-!!!\r");
    }
    UART_RX_buffer_size = 0;

}

unsigned int readLightSensor()
{
    unsigned int ADCresult;

    AD1CON1bits.ADON = 1;   // enable ADC
    AD1CON1bits.SAMP = 1;   // enable ADC
    IFS0bits.AD1IF = 0;
    while (!AD1CON1bits.DONE); // || AD1CON1bits.SAMP) { }
    //while(!IFS0bits.AD1IF);  //waiting
    //IFS0bits.AD1IF = 0;
    ADCresult = ADC1BUF0;

    AD1CON1bits.ADON = 0;

    //printf("L: %03X\r", ADCresult);
    //return ADCresult >> 4;
    return ADCresult;
}

void __attribute__((interrupt, no_auto_psv)) _ADC1Interrupt(void)
{
    //ADCresult = ADC1BUF0;
    IFS0bits.AD1IF = 0;     // clear ADC1 interrupt flag
}

float HTU21D_readTemp()
{
    // read the temperature from the HTU21D
    // uses the master hold command: takes about 40-50ms
    unsigned int rawTemp;
    float realTemp;
    unsigned char slaveAddress = 0x40;  // HTU21D fixed address
    unsigned char data[3];


    OpenI2C1( I2C_ON & I2C_IDLE_CON & I2C_CLK_HLD &
     I2C_IPMI_DIS & I2C_7BIT_ADD &
     I2C_SLW_DIS & I2C_SM_DIS &
     I2C_GCALL_DIS & I2C_STR_DIS &
     I2C_NACK & I2C_ACK_DIS & I2C_RCV_DIS &
     I2C_STOP_DIS & I2C_RESTART_DIS &
     I2C_START_DIS, I2C_BRG );

    IdleI2C1();		//Wait to complete
    StartI2C1();	//Send the Start Bit
    IdleI2C1();		//Wait to complete
    while(I2C1CONbits.SEN);
    //IFS1bits.MI2C1IF = 0;   //Clear interrupt flag

    MasterWriteI2C1((slaveAddress<<1)|0); //transmit write command
    IdleI2C1();		//Wait to complete

    MasterWriteI2C1(0xE3); //transmit command: read temperature (master hold)
    IdleI2C1();		//Wait to complete

    StartI2C1();	//Send the Start Bit
    IdleI2C1();		//Wait to complete
    while(I2C1CONbits.SEN); // Wait till Start sequence is completed

    MasterWriteI2C1((slaveAddress<<1)|1); //transmit read command
    IdleI2C1();		//Wait to complete

    // at this point the HTU21D will hold the clock low while it makes a measurement
    // takes about 40-50ms

    // read back 3 bytes
    //num = MastergetsI2C1(3, data, 40); // this doesn't work  (y not??)

    data[0] = MasterReadI2C1();     // read MSB
    AckI2C1();      // send I2C Ack
    IdleI2C1();     // wait
    data[1] = MasterReadI2C1();     // read LSB
    AckI2C1();
    IdleI2C1();		
    data[2] = MasterReadI2C1();     // read checksum
    AckI2C1();
    IdleI2C1();		

    StopI2C1();	//Send the Stop condition
    IdleI2C1();	//Wait to complete

    CloseI2C1();

    rawTemp = ((unsigned int) data[0] << 8) | (unsigned int) (data[1] & 0xFC);

    //Given the raw temperature data, calculate the actual temperature
    //temp = (rawTemp / (float)65536); //2^16 = 65536
    realTemp = -46.85 + (175.72 * ((float)rawTemp / (float)65536)); //From page 14 of HTU21D datasheet

    return realTemp;
}

float HTU21D_readHumidity()
{
    // read the humidity from the HTU21D
    // uses the master hold command: takes about 40-50ms
    unsigned int rawHumid;
    float realHumid;
    unsigned char slaveAddress = 0x40;  // HTU21D fixed address
    unsigned char data[3];

    OpenI2C1( I2C_ON & I2C_IDLE_CON & I2C_CLK_HLD &
     I2C_IPMI_DIS & I2C_7BIT_ADD &
     I2C_SLW_DIS & I2C_SM_DIS &
     I2C_GCALL_DIS & I2C_STR_DIS &
     I2C_NACK & I2C_ACK_DIS & I2C_RCV_DIS &
     I2C_STOP_DIS & I2C_RESTART_DIS &
     I2C_START_DIS, I2C_BRG );

    IdleI2C1();		//Wait to complete
    StartI2C1();	//Send the Start Bit
    IdleI2C1();		//Wait to complete
    while(I2C1CONbits.SEN);
    //IFS1bits.MI2C1IF = 0;   //Clear interrupt flag

    MasterWriteI2C1((slaveAddress<<1)|0); //transmit write command
    IdleI2C1();		//Wait to complete

    MasterWriteI2C1(0xE5); //transmit command: read humidity (master hold)
    IdleI2C1();		//Wait to complete

    StartI2C1();	//Send the Start Bit
    IdleI2C1();		//Wait to complete
    while(I2C1CONbits.SEN); // Wait till Start sequence is completed

    MasterWriteI2C1((slaveAddress<<1)|1); //transmit read command
    IdleI2C1();		//Wait to complete

    // at this point the HTU21D will hold the clock low while it makes a measurement
    // takes about 40-50ms

    // read back 3 bytes
    //num = MastergetsI2C1(3, data, 40); // this doesn't work  (y not??)

    data[0] = MasterReadI2C1();     // read MSB
    AckI2C1();      // send I2C Ack
    IdleI2C1();     // wait
    data[1] = MasterReadI2C1();     // read LSB
    AckI2C1();
    IdleI2C1();
    data[2] = MasterReadI2C1();     // read checksum
    AckI2C1();
    IdleI2C1();

    StopI2C1();	//Send the Stop condition
    IdleI2C1();	//Wait to complete

    CloseI2C1();

    rawHumid = ((unsigned int) data[0] << 8) | (unsigned int) (data[1] & 0xFC);

    //Given the raw temperature data, calculate the actual temperature
    //temp = (rawTemp / (float)65536); //2^16 = 65536
    realHumid = -6 + (125 * ((float)rawHumid / (float)65536)); //From page 14 of HTU21D datasheet

    return realHumid;
}